//----------------------------------------------------------------------------------
// File:   EffectWithOneTechnique.h
// Author: Kevin Myers
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

// This file contains a simple wrapper class that helps with effect creation.  It simply
// takes in the name D3D10 device, the name of the file, and the pool the effect will be
// added to.  It assumes there is only 1 technque per file, and also assumes all 
// all variables will be set through the pool since it doesn't expose the underlying 
// effect.

#pragma once

#include <crtdbg.h>
#include <d3d10.h>

class EffectWithOneTechnique
{
public:
    ID3D10EffectTechnique*    pTechnique;

    EffectWithOneTechnique( ID3D10Device* pD3D10Device, LPCWSTR pSrcFile , ID3D10EffectPool* pPool = NULL )
    {
        m_pEffect = NULL;
        pTechnique = NULL;
        m_pD3D10Device = pD3D10Device;

        //Create our effect
        WCHAR str[MAX_PATH];
		HRESULT hr;
        V( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, pSrcFile ) );
        D3DX10CreateEffectFromFile( str, NULL, NULL, "fx_4_0", 0, pPool ? D3D10_EFFECT_COMPILE_CHILD_EFFECT : NULL, pD3D10Device, pPool, NULL, &m_pEffect, NULL, &hr );
        _ASSERT( m_pEffect && m_pEffect->IsValid() );

        pTechnique = m_pEffect->GetTechniqueByIndex( 0 );
        _ASSERT( pTechnique->IsValid() );

        D3D10_TECHNIQUE_DESC techDesc;
        pTechnique->GetDesc( &techDesc );
        m_nPasses = techDesc.Passes;
        _ASSERT( m_nPasses < 100 && m_nPasses > 0 ); //Make sure this number isn't crazy

        //Cache pass descriptions
        m_pPassDescs = new D3D10_PASS_DESC[ m_nPasses ];
        for(UINT i = 0; i < m_nPasses; ++i )
            pTechnique->GetPassByIndex( i )->GetDesc( &m_pPassDescs[i] ); 
    }
    void CreateIA( UINT iPassIndex, const D3D10_INPUT_ELEMENT_DESC *pInputElementDescs, UINT NumElements, ID3D10InputLayout **ppInputLayout )
    {
        _ASSERT( ppInputLayout );
        m_pD3D10Device->CreateInputLayout( pInputElementDescs, NumElements, m_pPassDescs[iPassIndex].pIAInputSignature, m_pPassDescs[iPassIndex].IAInputSignatureSize, ppInputLayout );
        _ASSERT( *ppInputLayout );
    }
    ID3D10EffectTechnique* GetTechnique(UINT i)
    {
        return m_pEffect->GetTechniqueByIndex( i );
    }
    void ApplyPass( UINT i )
    {
        pTechnique->GetPassByIndex( i )->Apply( 0 );
    }
    ~EffectWithOneTechnique()
    {
        SAFE_RELEASE( m_pEffect );
        SAFE_DELETE( m_pPassDescs );
    }

protected:
    ID3D10Effect*            m_pEffect;
    D3D10_PASS_DESC*        m_pPassDescs;

    ID3D10Device*            m_pD3D10Device;
    UINT                    m_nPasses;
private:
};    
